import dogpile.util.compat
import hashlib
import inspect


def key_generator(ns, fn, to_str=dogpile.util.compat.text_type):
    """Extension of dogpile.util.function_key_generator that handles
    non-ascii function arguments, and supports not just plain functions, but
    methods as well.
    """
    args = inspect.getargspec(fn)
    has_self = args[0] and args[0][0] in ('self', 'cls')

    def generate_key(*args, **kw):
        if kw:
            raise ValueError(
                "dogpile.cache's default key creation "
                "function does not accept keyword arguments.")
        if has_self and args:
            cls = args[0].__class__
            args = args[1:]
        else:
            cls = None

        if cls:
            namespace = u'.'.join([cls.__module__, cls.__name__, fn.__name__])
        else:
            namespace = u'.'.join([fn.__module__, fn.__name__])
        if ns is not None:
            namespace = u'%s|%s' % (namespace, ns)

        return namespace + u'|' + u' '.join(map(to_str, args))
    return generate_key


def sha1_mangle_key(key):
    """Extension of dogpile.util.sha1_mangle_key that handles unicode."""
    return hashlib.sha1(key.encode('utf-8')).hexdigest()



from dogpile.cache.backends.memory import MemoryBackend
from pyramid_dogpile_cache2 import configure_dogpile_cache, get_region
import dogpile.cache.backends.memcached
import dogpile.cache.exception
import pytest


def test_sets_backend_for_regions(empty_config):
    configure_dogpile_cache({
        'dogpile_cache.regions': 'foo, bar',
        'dogpile_cache.backend': 'dogpile.cache.memory',
        'dogpile_cache.expiration_time': '1',
    })
    assert isinstance(get_region('foo').backend, MemoryBackend)
    assert isinstance(get_region('bar').backend, MemoryBackend)


def test_converts_expiration_time(empty_config):
    configure_dogpile_cache({
        'dogpile_cache.regions': 'foo',
        'dogpile_cache.backend': 'dogpile.cache.memory',
        'dogpile_cache.expiration_time': '1',
    })
    assert get_region('foo').expiration_time == 1


def test_converts_expiration_time_to_int(empty_config):
    configure_dogpile_cache({
        'dogpile_cache.regions': 'foo',
        'dogpile_cache.backend': 'dogpile.cache.memory',
        'dogpile_cache.expiration_time': '1',
    })
    assert get_region('foo').expiration_time == 1


def test_sets_memcache_expire_time_to_later_than_expiration_time(
        empty_config, monkeypatch):
    monkeypatch.setattr(
        dogpile.cache.backends.memcached.PylibmcBackend, '_imports',
        lambda self: None)
    configure_dogpile_cache({
        'dogpile_cache.regions': 'foo',
        'dogpile_cache.backend': 'dogpile.cache.pylibmc',
        'dogpile_cache.pylibmc_url': 'http://localhost:8899',
        'dogpile_cache.expiration_time': '1',
        'dogpile_cache.memcache_expire_time_interval': '5'
    })
    assert get_region('foo').backend.memcached_expire_time == 6


def test_sets_pylibmc_behaviours(empty_config, monkeypatch):
    monkeypatch.setattr(
        dogpile.cache.backends.memcached.PylibmcBackend, '_imports',
        lambda self: None)
    configure_dogpile_cache({
        'dogpile_cache.regions': 'foo',
        'dogpile_cache.backend': 'dogpile.cache.pylibmc',
        'dogpile_cache.pylibmc_url': 'http://localhost:8899',
        'dogpile_cache.expiration_time': '1',
        'dogpile_cache.pylibmc_behavior.send_timeout': '5',
    })
    assert get_region('foo').backend.behaviors == {'send_timeout': 5}


def test_unconfigured_region_raises(empty_config):
    get_region('bar')
    with pytest.raises(dogpile.cache.exception.RegionNotConfigured):
        configure_dogpile_cache({
            'dogpile_cache.regions': 'foo',
            'dogpile_cache.backend': 'dogpile.cache.memory',
            'dogpile_cache.expiration_time': '1',
        })



# coding: utf-8
from pyramid_dogpile_cache2 import get_region
import dogpile.util.compat
import dogpile.cache.util
import pyramid_dogpile_cache2.cache
import pytest


@pytest.mark.skipif(
    dogpile.util.compat.py3k,
    reason='dogpile.cache defines string_type=str in both py2 and py3, '
    'which is not the same thing at all, thus this only breaks in py2')
def test_key_generator_handles_non_ascii_arguments():
    gen = dogpile.cache.util.function_key_generator(None, lambda: None)
    with pytest.raises(UnicodeEncodeError):
        gen(u'föö')

    gen = pyramid_dogpile_cache2.cache.key_generator(None, lambda: None)
    gen(u'föö')


def test_mangle_key_handles_non_ascii_arguments():
    with pytest.raises(Exception):
        dogpile.cache.util.sha1_mangle_key(u'föö')

    pyramid_dogpile_cache2.cache.sha1_mangle_key(u'föö')


def test_key_generator_adds_class_name_for_methods():
    def plain_function():
        pass

    class Foo(object):

        def method(self):
            pass

    gen = dogpile.cache.util.function_key_generator(None, plain_function)
    assert gen() == 'pyramid_dogpile_cache2.tests.test_cache:plain_function|'
    gen = dogpile.cache.util.function_key_generator(None, Foo().method)
    assert gen() == 'pyramid_dogpile_cache2.tests.test_cache:method|'

    gen = pyramid_dogpile_cache2.cache.key_generator(None, plain_function)
    assert gen() == 'pyramid_dogpile_cache2.tests.test_cache.plain_function|'
    foo = Foo()
    gen = pyramid_dogpile_cache2.cache.key_generator(None, foo.method)
    assert gen(foo) == 'pyramid_dogpile_cache2.tests.test_cache.Foo.method|'


def test_methods_are_detected_when_decorated(empty_config):
    region = get_region('test')
    region.configure('dogpile.cache.memory')

    class Foo(object):

        @region.cache_on_arguments()
        def method(self):
            return 42

    Foo().method()
    assert region.get('pyramid_dogpile_cache2.tests.test_cache.Foo.method|') == 42



"""Integrates dogpile.cache for Pyramid
"""
from setuptools import setup, find_packages
import os.path


def project_path(*names):
    return os.path.join(os.path.dirname(__file__), *names)


setup(
    name='pyramid_dogpile_cache2',
    version='1.0.3',

    install_requires=[
        'Beaker',  # For parsing pylibmc behaviors from ini file.
        'dogpile.cache >= 0.6.0.dev0',
        'pyramid_dogpile_cache',  # For ini file parsing helpers.
        'setuptools',
    ],

    author='Zeit Online',
    author_email='zon-backend@zeit.de',
    license='BSD',
    url='https://github.com/zeitonline/pyramid_dogpile_cache2',

    keywords='pyramid dogpile.cache',
    classifiers="""\
    Environment :: Plugins
    Framework :: Pyramid
    Framework :: Paste
    Intended Audience :: Developers
    License :: OSI Approved :: BSD License
    Programming Language :: Python
    Programming Language :: Python :: 2
    Programming Language :: Python :: 2.7
    Programming Language :: Python :: 3
    Programming Language :: Python :: 3.4
    """.split('\n')[:-1],
    description=__doc__.strip(),
    long_description='\n\n'.join(open(project_path(name)).read() for name in (
        'README.rst',
        'HACKING.rst',
        'CHANGES.txt',
    )),

    packages=find_packages('src'),
    package_dir={'': 'src'},
    include_package_data=True,
    zip_safe=False,
)
